@@ -6,7 +6,7 @@
 //  |  Y Y  \|  |  /|    |     / __ \_|  | \/\___ \ \  ___/ |  | \/
 //  |__|_|  /|____/ |____|    (____  /|__|  /____  > \___  >|__|   
 //        \/                       \/            \/      \/        
-//  (C) 2011 Ingo Berg
+//  (C) 2013 Ingo Berg
 //
 //  example1.cpp - using the parser as a static library
 //
@@ -80,8 +80,9 @@ value_type ThrowAnException(value_type)
 //---------------------------------------------------------------------------
 value_type BulkFun1(int nBulkIdx, int nThreadIdx, value_type v1)
 {
-  int nThread = nThreadIdx; 
-  return nBulkIdx + v1;
+  // Note: I'm just doing something with all three parameters to shut 
+  // compiler warnings up!
+  return nBulkIdx + nThreadIdx + v1;
 }
 
 //---------------------------------------------------------------------------
@@ -98,7 +99,7 @@ value_type StrFun0(const char_type *szMsg)
     mu::console() << szMsg << std::endl;
 
   return 999;
-};
+}
 
 //---------------------------------------------------------------------------
 value_type StrFun2(const char_type *v1, value_type v2,value_type v3) 
@@ -125,64 +126,25 @@ value_type* AddVariable(const char_type *a_szName, void *a_pUserData)
   // in order to delete them later on. Or you find other ways to keep track of 
   // variables that have been created implicitely.
   static value_type afValBuf[100];  
-  static int iVal = 0;          
+  static int iVal = -1;
+
+  ++iVal;
 
   mu::console() << _T("Generating new variable \"") 
-                << a_szName << _T("\" (slots left: ")
+                << a_szName << std::dec << _T("\" (slots left: ")
                 << 99-iVal << _T(")")
                 << _T(" User data pointer is:") 
                 << std::hex << a_pUserData <<endl;
-
   afValBuf[iVal] = 0;
+
   if (iVal>=99)
     throw mu::ParserError( _T("Variable buffer overflow.") );
-
-  return &afValBuf[iVal++];
+  else
+    return &afValBuf[iVal];
 }
 
 int IsHexValue(const char_type *a_szExpr, int *a_iPos, value_type *a_fVal) 
 { 
-  //My method, but will use the similar one implemented on muParserInt.cpp 
-  /*
-  if (a_szExpr[0]!='0' && a_szExpr[1]!='x') 
-    return 0; 
-	
-  unsigned int maxSizeToCheckAsHex = 32; 
-  unsigned int charValue = 0; 
-  unsigned int charPosition; 
-  
-  for (charPosition = 0; (((a_szExpr[charPosition+2] >= '0' && a_szExpr[charPosition+2] <= '9') || 
-                           (a_szExpr[charPosition+2] >= 'a' && a_szExpr[charPosition+2] <= 'f') || 
-						   (a_szExpr[charPosition+2] >= 'A' && a_szExpr[charPosition+2] <= 'F')) && 
-						   (charPosition < maxSizeToCheckAsHex)); charPosition++) 
-  { 
-    if(a_szExpr[charPosition+2] >= '0' && a_szExpr[charPosition+2] <= '9') 
-	{ 
-	  charValue = ((charValue*16) + (a_szExpr[charPosition+2] - 48)); 
-	} 
-	else if (a_szExpr[charPosition+2] >= 'a' && a_szExpr[charPosition+2] <= 'f') 
-	{ 
-	  charValue = ((charValue*16) + (a_szExpr[charPosition+2] - 87)); 
-	} 
-	else if(a_szExpr[charPosition+2] >= 'A' && a_szExpr[charPosition+2] <= 'F') 
-	{ 
-	  charValue = ((charValue*16) + (a_szExpr[charPosition+2] - 55)); 
-	} 
-  } 
-  
-  if (charPosition==0) 
-    return 0; 
-	
-  if (charPosition == maxSizeToCheckAsHex) 
-    throw mu::Parser::exception_type("Binary to integer conversion error (overflow)."); 
-	
-  *a_fVal = (value_type)charValue; 
-  *a_iPos += charPosition+1; 
-  return 1;
-*/ 
-
-//*************Method 2****************** 
-
   if (a_szExpr[1]==0 || (a_szExpr[0]!='0' || a_szExpr[1]!='x') ) 
     return 0;
 
@@ -199,6 +161,7 @@ int IsHexValue(const char_type *a_szExpr, int *a_iPos, value_type *a_fVal)
 
   *a_iPos += (int)(2 + nPos);
   *a_fVal = (value_type)iVal;
+
   return 1;
 }
 
@@ -212,7 +175,7 @@ void Splash()
   mu::console() << _T("  |__|_|  /|____/ |____|    (____  /|__|  /____  > \\___  >|__|    \n");
   mu::console() << _T("        \\/                       \\/            \\/      \\/         \n");
   mu::console() << _T("  Version ") << Parser().GetVersion(pviFULL) << _T("\n");
-  mu::console() << _T("  (C) 2011 Ingo Berg\n");
+  mu::console() << _T("  (C) 2013 Ingo Berg\n");
 }
 
 //---------------------------------------------------------------------------
@@ -243,6 +206,8 @@ value_type Help()
   mu::console() << _T( "  list var     - list parser variables\n");
   mu::console() << _T( "  list exprvar - list expression variables\n");
   mu::console() << _T( "  list const   - list all numeric parser constants\n");
+  mu::console() << _T( "  opt on       - enable optimizer (default)\n");
+  mu::console() << _T( "  opt off      - disable optimizer\n");
   mu::console() << _T( "  locale de    - switch to german locale\n");
   mu::console() << _T( "  locale en    - switch to english locale\n");
   mu::console() << _T( "  locale reset - reset locale\n");
@@ -384,6 +349,18 @@ int CheckKeywords(const mu::char_type *a_szLine, mu::Parser &a_Parser)
     ListVar(a_Parser);
     return 1;
   }
+  else if ( sLine == _T("opt on") )
+  {
+    a_Parser.EnableOptimizer(true);
+    mu::console() << _T("Optimizer enabled\n");
+    return 1;
+  }
+  else if ( sLine == _T("opt off") )
+  {
+    a_Parser.EnableOptimizer(false);
+    mu::console() << _T("Optimizer disabled\n");
+    return 1;
+  }
   else if ( sLine == _T("list const") )
   {
     ListConst(a_Parser);
@@ -503,10 +480,10 @@ void Calc()
   parser.DefinePostfixOprt(_T("M"), Mega);
   parser.DefinePostfixOprt(_T("m"), Milli);
   parser.DefineInfixOprt(_T("!"), Not);
-  parser.DefineFun(_T("strfun0"), StrFun0, false);
-  parser.DefineFun(_T("strfun2"), StrFun2, false);
-  parser.DefineFun(_T("ping"), Ping, false);
-  parser.DefineFun(_T("rnd"), Rnd, false);     // Add an unoptimizeable function
+  parser.DefineFun(_T("strfun0"), StrFun0);
+  parser.DefineFun(_T("strfun2"), StrFun2);
+  parser.DefineFun(_T("ping"), Ping);
+  parser.DefineFun(_T("rnd"), Rnd);     // Add an unoptimizeable function
   parser.DefineFun(_T("throw"), ThrowAnException);
 
   
